﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Xml;

namespace StudyCommerceTimerTask.TimerTask {

	/// <summary>
	/// 需要执行的任务 从config 读取配置信息
	/// Yuhao 2014.5.6 Add
	/// </summary>
	public partial class Task {

		#region Fields

		/// <summary>
		/// 任务接口
		/// </summary>
		private ITask _task;

		/// <summary>
		/// 是否开启任务
		/// </summary>
		private bool _enabled;

		/// <summary>
		/// 任务类型
		/// </summary>
		private Type _taskType;

		/// <summary>
		/// 任务名称
		/// </summary>
		private string _name;

		/// <summary>
		/// 遇见错误是否停止
		/// </summary>
		private bool _stopOnError;

		/// <summary>
		/// config的配置节点
		/// </summary>
		private XmlNode _configNode;

		/// <summary>
		/// 上次执行开始时间
		/// </summary>
		private DateTime _lastStarted;

		/// <summary>
		/// 上次执行成功时间
		/// </summary>
		private DateTime _lastSuccess;

		/// <summary>
		/// 上次执行结束时间
		/// </summary>
		private DateTime _lastEnd;

		/// <summary>
		/// 是否启动运行
		/// </summary>
		private bool _isRunning;

		#endregion

		#region .ctor
		/// <summary>
		/// 无参的私有构造方法
		/// </summary>
		private Task()
		{
			this._enabled = true;
		}
		/// <summary>
		/// 带参数的公共构造函数
		/// </summary>
		/// <param name="taskType">任务类型</param>
		/// <param name="configNode">配置节点</param>
		public Task(Type taskType, XmlNode configNode)
		{
			this._enabled = true;
			this._configNode = configNode;
			this._taskType = taskType;
			
			//获取节点值，当不为空或者转换失败的时候 默认为True
			if ((configNode.Attributes["enabled"] != null) &&
			    !bool.TryParse(configNode.Attributes["enabled"].Value, out this._enabled))
			{
				this._enabled = true;
			}
			if ((configNode.Attributes["stopOnError"] != null) &&
			    !bool.TryParse(configNode.Attributes["stopOnError"].Value, out this._stopOnError))
			{
				this._stopOnError = true;
			}
			if (configNode.Attributes["name"] != null)
			{
				this._name = configNode.Attributes["name"].Value;
			}
		}
		#endregion

		#region Methods

		public ITask createTask()
		{
			if (this.Enabled && (this._task == null))
			{
				if (this._taskType != null)
				{
					this._task = Activator.CreateInstance(this._taskType) as ITask;
				}
			}
			return this._task;
		}

		public void Execute()
		{
			this._isRunning = true;
			try
			{
				var task = this.createTask();
				if (task != null)
				{
					this._lastStarted = DateTime.Now;
					task.Execute(this._configNode);
					this._lastEnd = this._lastSuccess = DateTime.Now;
				}
			}
			catch (Exception exception)
			{
				this._enabled = !this.StopOnError;
				this._lastEnd = DateTime.Now;
				

				//throw;
			}
			this._isRunning = false;
		}
		#endregion

		#region Properties

		/// <summary>
		/// 指示任务是否在执行
		/// </summary>
		public bool IsRunning
		{
			get
			{
				return this._isRunning;
			}
		}
		/// <summary>
		/// 上次开始时间
		/// </summary>
		public DateTime LastStarted
		{
			get
			{
				return this._lastStarted;
			}
		}

		/// <summary>
		/// 上次执行成功时间
		/// </summary>
		public DateTime LastSuccess
		{
			get
			{
				return this._lastStarted;
			}
		}

		/// <summary>
		/// 上次执行结束时间
		/// </summary>
		public DateTime LastEnd
		{
			get
			{
				return this._lastEnd;
			}
		}

		/// <summary>
		/// 任务类型
		/// </summary>
		public Type TaskType
		{
			get
			{
				return this._taskType;
			}
		}

		/// <summary>
		/// 是否停止当差生错误
		/// </summary>
		public bool StopOnError
		{
			get
			{
				return this._stopOnError;
			}
		}

		/// <summary>
		/// 名称
		/// </summary>
		public string Name
		{
			get
			{
				return this._name;
			}
		}

		/// <summary>
		/// 是否启用
		/// </summary>
		public bool Enabled
		{
			get
			{
				return this._enabled;
			}
		}
		#endregion
	}
}